@@ -2,15 +2,15 @@
 """
 
 # Create an image stack with random data
-im = signals.Image({'data' : np.random.random((16,32,32))})
+im = signals.Image(np.random.random((16,32,32)))
 
 # Define the axis properties
-im.axes_manager.signal_axes[0].name = 'Y'
+im.axes_manager.signal_axes[0].name = 'X'
 im.axes_manager.signal_axes[0].units = 'nm'
 im.axes_manager.signal_axes[0].scale = 0.1
 im.axes_manager.signal_axes[0].offset = 0
 
-im.axes_manager.signal_axes[1].name = 'X'
+im.axes_manager.signal_axes[1].name = 'Y'
 im.axes_manager.signal_axes[1].units = 'nm'
 im.axes_manager.signal_axes[1].scale = 0.1
 im.axes_manager.signal_axes[1].offset = 0
