@@ -2,7 +2,7 @@
 """
 
 # Create a 2D image stack with random data
-im = signals.Image({'data' : np.random.random((16,16,32,32))})
+im = signals.Image(np.random.random((16,16,32,32)))
 
 # Define the axis properties
 im.axes_manager.signal_axes[0].name = ''
@@ -15,12 +15,12 @@ im.axes_manager.signal_axes[1].units = '1/nm'
 im.axes_manager.signal_axes[1].scale = 0.1
 im.axes_manager.signal_axes[1].offset = 0
 
-im.axes_manager.navigation_axes[0].name = 'Y'
+im.axes_manager.navigation_axes[0].name = 'X'
 im.axes_manager.navigation_axes[0].units = 'nm'
 im.axes_manager.navigation_axes[0].scale = 0.3
 im.axes_manager.navigation_axes[0].offset = 100
 
-im.axes_manager.navigation_axes[1].name = 'X'
+im.axes_manager.navigation_axes[1].name = 'Y'
 im.axes_manager.navigation_axes[1].units = 'nm'
 im.axes_manager.navigation_axes[1].scale = 0.3
 im.axes_manager.navigation_axes[1].offset = 100
