@@ -2,7 +2,7 @@
 """
 
 # Create a spectrum image with random data
-s = signals.Spectrum({'data' : np.random.random((64, 64, 1024))})
+s = signals.Spectrum(np.random.random((64, 64, 1024)))
 
 # Define the axis properties
 s.axes_manager.signal_axes[0].name = 'Energy'
@@ -10,12 +10,12 @@ s.axes_manager.signal_axes[0].units = 'eV'
 s.axes_manager.signal_axes[0].scale = 0.3
 s.axes_manager.signal_axes[0].offset = 100
 
-s.axes_manager.navigation_axes[0].name = 'Y'
+s.axes_manager.navigation_axes[0].name = 'X'
 s.axes_manager.navigation_axes[0].units = 'nm'
 s.axes_manager.navigation_axes[0].scale = 0.1
 s.axes_manager.navigation_axes[0].offset = 100
 
-s.axes_manager.navigation_axes[1].name = 'X'
+s.axes_manager.navigation_axes[1].name = 'Y'
 s.axes_manager.navigation_axes[1].units = 'nm'
 s.axes_manager.navigation_axes[1].scale = 0.1
 s.axes_manager.navigation_axes[1].offset = 100
