@@ -16,11 +16,14 @@
 # You should have received a copy of the GNU General Public License
 # along with  Hyperspy.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import division
+import copy
+
 import numpy as np
 
-import widgets
-import spectrum, image, utils
-import copy
+from hyperspy.drawing import widgets, spectrum, image, utils
+from hyperspy.gui.axes import navigation_sliders
+
 
 class MPL_HyperSpectrum_Explorer(object):
     """Plots the current spectrum to the screen and a map with a cursor 
@@ -38,16 +41,12 @@ class MPL_HyperSpectrum_Explorer(object):
         self.navigator_title = ''
         self.navigator_plot = None
         self.signal_plot = None
-        self.pixel_size = None
-        self.pixel_units =  None
         self.axis = None
         self.pointer = None
         self.right_pointer = None
         self._key_nav_cid = None
         self._right_pointer_on = False
         self._auto_update_plot = True
-        self.plot_navigator_scalebar = False
-        self.plot_navigator_plot_ticks = True
         
     @property
     def auto_update_plot(self):
@@ -85,7 +84,7 @@ class MPL_HyperSpectrum_Explorer(object):
     
     def assign_pointer(self):
         nav_dim = self.axes_manager.navigation_dimension
-        if nav_dim == 2:
+        if nav_dim >= 2:
             Pointer = widgets.DraggableSquare
         elif nav_dim == 1:
             Pointer = widgets.DraggableHorizontalLine
@@ -94,7 +93,6 @@ class MPL_HyperSpectrum_Explorer(object):
         return Pointer
    
     def plot(self):
-        self.generate_labels()
         if self.pointer is None:
             pointer = self.assign_pointer()  
             if pointer is not None:
@@ -104,60 +102,26 @@ class MPL_HyperSpectrum_Explorer(object):
             self.plot_navigator()
         self.plot_signal()
         
-    def generate_labels(self):
-        # Spectrum plot labels
-        self.xlabel = '%s (%s)' % (
-                self.axes_manager.signal_axes[0].name,
-                self.axes_manager.signal_axes[0].units)
-        self.ylabel = 'Intensity'
-        self.axis = self.axes_manager.signal_axes[0].axis
-        # Navigator labels
-        if self.axes_manager.navigation_dimension == 1:
-            scalebar_axis = self.axes_manager.signal_axes[0]
-            self.navigator_xlabel = '%s (%s)' % (
-                self.axes_manager.signal_axes[0].name,
-                self.axes_manager.signal_axes[0].units)
-            self.navigator_ylabel = '%s (%s)' % (
-                self.axes_manager.navigation_axes[0].name,
-                self.axes_manager.navigation_axes[0].units)
-            self.plot_navigator_scalebar = False
-            self.plot_navigator_ticks = True
-            self.pixel_units = scalebar_axis.units
-            self.pixel_size = scalebar_axis.scale
-
-
-        elif self.axes_manager.navigation_dimension == 2:
-            scalebar_axis = \
-                self.axes_manager.navigation_axes[-1]
-            self.navigator_ylabel = '%s (%s)' % (
-                self.axes_manager.navigation_axes[0].name,
-                self.axes_manager.navigation_axes[0].units)
-            self.navigator_xlabel = '%s (%s)' % (
-                self.axes_manager.navigation_axes[1].name,
-                self.axes_manager.navigation_axes[1].units)
-            if (self.axes_manager.navigation_axes[0].units == 
-                self.axes_manager.navigation_axes[1].units):
-                    self.plot_navigator_scalebar = True
-                    self.plot_navigator_ticks = False
-            else:
-                self.plot_navigator_scalebar = False
-                self.plot_navigator_ticks = True
-            self.pixel_units = scalebar_axis.units
-            self.pixel_size = scalebar_axis.scale
-        
-        
     def plot_navigator(self):
         if self.navigator_plot is not None:
             self.navigator_plot.plot()
             return
         imf = image.ImagePlot()
         imf.data_function = self.navigator_data_function
-        imf.pixel_units = self.pixel_units
-        imf.pixel_size = self.pixel_size
-        imf.xlabel = self.navigator_xlabel
-        imf.ylabel = self.navigator_ylabel
-        imf.plot_scalebar = self.plot_navigator_scalebar
-        imf.plot_ticks = self.plot_navigator_ticks
+        # Navigator labels
+        if self.axes_manager.navigation_dimension == 1:
+            imf.yaxis = self.axes_manager.navigation_axes[0]
+            imf.xaxis = self.axes_manager.signal_axes[0]
+        elif self.axes_manager.navigation_dimension >= 2:
+            imf.yaxis = self.axes_manager.navigation_axes[1]
+            imf.xaxis = self.axes_manager.navigation_axes[0]
+            if self.axes_manager.navigation_dimension > 2:
+                navigation_sliders(
+                    self.axes_manager.navigation_axes)
+                for axis in self.axes_manager.navigation_axes[2:]:
+                    axis.connect(imf.update_image)
+            
+        imf.title = self.signal_title + ' Navigator'
         imf.plot()
         self.pointer.add_axes(imf.ax)
         self.navigator_plot = imf
@@ -166,8 +130,12 @@ class MPL_HyperSpectrum_Explorer(object):
         if self.signal_plot is not None:
             self.signal_plot.plot()
             return
-        
         # Create the figure
+        self.xlabel = '%s (%s)' % (
+            self.axes_manager.signal_axes[0].name,
+            self.axes_manager.signal_axes[0].units)
+        self.ylabel = 'Intensity'
+        self.axis = self.axes_manager.signal_axes[0].axis
         sf = spectrum.SpectrumFigure()
         sf.xlabel = self.xlabel
         sf.ylabel = self.ylabel
@@ -176,23 +144,34 @@ class MPL_HyperSpectrum_Explorer(object):
         sf.create_axis()
         sf.axes_manager = self.axes_manager
         self.signal_plot = sf
-        # Create a line to the left axis with the default coordinates
+        # Create a line to the left axis with the default indices
         sl = spectrum.SpectrumLine()
         sl.data_function = self.signal_data_function
-        sl.plot_coordinates = True
+        sl.plot_indices = True
         if self.pointer is not None:
             color = self.pointer.color
         else:
             color = 'red'
         sl.line_properties_helper(color, 'step')        
         # Add the line to the figure
-          
         sf.add_line(sl)
+        # If the data is complex create a line in the left axis with the
+        # default coordinates
+        sl = spectrum.SpectrumLine()
+        sl.data_function = self.signal_data_function
+        sl.plot_coordinates = True
+        sl.get_complex = any(np.iscomplex(sl.data_function()))        
+        if sl.get_complex:
+            sl.line_properties_helper("blue", 'step')        
+            # Add extra line to the figure
+            sf.add_line(sl)
+        
+        
         self.signal_plot = sf
         sf.plot()
         if self.navigator_plot is not None and sf.figure is not None:
             utils.on_figure_window_close(self.navigator_plot.figure, 
-            self._close_pointer)
+            self._disconnect)
             utils.on_figure_window_close(sf.figure,
                                             self.close_navigator_plot)
             self._key_nav_cid = \
@@ -209,7 +188,7 @@ class MPL_HyperSpectrum_Explorer(object):
                 'key_press_event', self.key2switch_right_pointer)
     
     def close_navigator_plot(self):
-        self._close_pointer()
+        self._disconnect()
         self.navigator_plot.close()
                 
     def key2switch_right_pointer(self, event):
@@ -232,7 +211,7 @@ class MPL_HyperSpectrum_Explorer(object):
         rl.line_properties_helper(self.right_pointer.color, 'step')
         self.signal_plot.create_right_axis()
         self.signal_plot.add_line(rl, ax = 'right')
-        rl.plot_coordinates = True
+        rl.plot_indices = True
         rl.text_position = (1., 1.05,)
         rl.plot()
         self.right_pointer_on = True
@@ -245,11 +224,16 @@ class MPL_HyperSpectrum_Explorer(object):
         self.right_pointer = None
         self.navigator_plot.update_image()
         
-    def _close_pointer(self):
+    def _disconnect(self):
+        if (self.axes_manager.navigation_dimension > 2 and 
+            self.navigator_plot is not None):
+                for axis in self.axes_manager.navigation_axes[:-2]:
+                    axis.disconnect(self.navigator_plot.update_image)
+        
         if self.pointer is not None:
             self.pointer.disconnect(self.navigator_plot.ax)
     
     def close(self):
-        self._close_pointer()
+        self._disconnect()
         self.signal_plot.close()
         self.navigator_plot.close()
