@@ -47,9 +47,9 @@ def _plot_quiver_scatter_overlay(image, axes_manager,
     """
     if ax==None:
         ax=plt.gca()
-    axes=axes_manager.signal_axes
+    axes=axes_manager.signal_axes[::-1]
     if len(axes)<2:
-        axes=axes_manager.axes
+        axes=axes_manager._axes
         if axes[0].index_in_array==0:
             axes=axes[0],axes[1]
     extent=None
@@ -117,8 +117,8 @@ def _plot_2D_component(factors, idx, axes_manager,
                        ):
     if ax==None:
         ax=plt.gca()
-    axes=axes_manager.signal_axes
-    shape=axes_manager.signal_shape
+    axes=axes_manager.signal_axes[::-1]
+    shape=axes_manager._signal_shape_in_array
     extent=None
     if calibrate:
         extent=(axes[1].low_value,
@@ -171,17 +171,18 @@ def _plot_loading(loadings, idx, axes_manager, ax=None,
     if axes_manager.navigation_dimension==2:
         extent=None
         # get calibration from a passed axes_manager
-        shape=axes_manager.navigation_shape
+        shape=axes_manager._navigation_shape_in_array
         if calibrate:
-            extent=(axes[1].low_value,
-                    axes[1].high_value,
+            extent=(axes[0].low_value,
                     axes[0].high_value,
-                    axes[0].low_value)
-        im=ax.imshow(loadings[idx].reshape(shape),cmap=cmap,extent=extent,
+                    axes[1].high_value,
+                    axes[1].low_value)
+        im=ax.imshow(loadings[idx].reshape(shape),
+                     cmap=cmap,extent=extent,
                      interpolation='nearest')
         if calibrate:
-            plt.xlabel(axes[1].units)
-            plt.ylabel(axes[0].units)
+            plt.xlabel(axes[0].units)
+            plt.ylabel(axes[1].units)
         else:
             plt.xlabel('pixels')
             plt.ylabel('pixels')
