@@ -23,6 +23,7 @@
 
 import codecs
 import os.path
+from StringIO import StringIO
 
 import numpy as np
 
@@ -94,10 +95,34 @@ rpl_keys = {
     'beam-energy' : float,
     }
 
+def correct_INCA_format(fp):
+    fp_list = list()
+    fp.seek(0)    
+    if '(' in fp.readline():
+        for line in fp:
+            line=line.replace("(MLX::","").replace(" : ", "\t").replace(" :", "\t").replace(" ","\t").lower().strip().replace(")","\n")
+            if "record-by" in line:
+                if "image" in line:
+                    line = "record-by\timage"
+                if "vector" in line:
+                    line = "record-by\tvector"
+                if "dont-care" in line:
+                    line = "record-by\tdont-care"                    
+            fp_list.append(line)
+        fp = StringIO()
+        fp.writelines(fp_list)
+    fp.seek(0)
+    return fp
+            
+
 def parse_ripple(fp):
     """Parse information from ripple (.rpl) file.
     Accepts file object 'fp. Returns dictionary rpl_info.
     """
+    
+
+    fp = correct_INCA_format(fp)
+                            
     rpl_info = {}
     for line in fp.readlines():
         line = line.replace(' ', '')
@@ -105,7 +130,7 @@ def parse_ripple(fp):
             line = line.strip('\r\n')
             #line = line.lower()
             if comment in line:
-                line = line[:line.find(comment)]
+                line = line[:line.find(comment)]           
             if not sep in line:
                 err = 'Separator in line "%s" is wrong, ' % line
                 err += 'it should be a <TAB> ("\\t")'
@@ -469,8 +494,8 @@ def file_writer(filename, signal, encoding='latin-1', *args, **kwds):
         depth_axis = signal.axes_manager.signal_axes[0]
         ev_per_chan = int(round(depth_axis.scale))
         if dimension == 3:
-            width_axis = signal.axes_manager.navigation_axes[1]
-            height_axis = signal.axes_manager.navigation_axes[0]
+            width_axis = signal.axes_manager.navigation_axes[0]
+            height_axis = signal.axes_manager.navigation_axes[1]
             depth, width, height = \
                             depth_axis.size, width_axis.size, height_axis.size
         elif dimension == 2:
@@ -481,8 +506,8 @@ def file_writer(filename, signal, encoding='latin-1', *args, **kwds):
             depth, width, height = depth_axis.size, 1, 1
 
     elif signal.axes_manager.signal_dimension == 2:
-        width_axis = signal.axes_manager.signal_axes[1]
-        height_axis = signal.axes_manager.signal_axes[0]
+        width_axis = signal.axes_manager.signal_axes[0]
+        height_axis = signal.axes_manager.signal_axes[1]
         if dimension == 3:
             depth_axis = signal.axes_manager.navigation_axes[0]
             record_by = 'image'
